package com.tzld.longarticle.recommend.server.service.recommend.filter.strategy;

import com.google.common.util.concurrent.ThreadFactoryBuilder;
import com.tzld.longarticle.recommend.server.common.CommonThreadPoolExecutor;
import com.tzld.longarticle.recommend.server.mapper.aigc.AigcBaseMapper;
import com.tzld.longarticle.recommend.server.model.dto.Content;
import com.tzld.longarticle.recommend.server.model.entity.aigc.PublishContent;
import com.tzld.longarticle.recommend.server.service.recommend.filter.FilterParam;
import com.tzld.longarticle.recommend.server.service.recommend.filter.FilterResult;
import com.tzld.longarticle.recommend.server.service.recommend.filter.FilterStrategy;
import com.tzld.longarticle.recommend.server.util.DateUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Collections;
import java.util.List;
import java.util.concurrent.*;
import java.util.stream.Collectors;

@Component
@Slf4j
public class TodayPublishStrategy implements FilterStrategy {

    @Autowired
    private AigcBaseMapper aigcBaseMapper;

    private final static ExecutorService pool = new CommonThreadPoolExecutor(
            8,
            8,
            0L, TimeUnit.SECONDS,
            new LinkedBlockingQueue<>(100),
            new ThreadFactoryBuilder().setNameFormat("TodayPublishStrategy-%d").build(),
            new ThreadPoolExecutor.AbortPolicy());

    @Override
    public FilterResult filter(FilterParam param) {
        Long start = System.currentTimeMillis();
        FilterResult filterResult = new FilterResult();
        List<String> result = Collections.synchronizedList(param.getContents().stream().map(Content::getId).collect(Collectors.toList()));
        Long todayStart = DateUtils.getTodayStart();
        List<PublishContent> todayPublishContentList = aigcBaseMapper.getTodayPublishContentList(param.getAccountId(), todayStart);
        if (CollectionUtils.isNotEmpty(todayPublishContentList)) {
            List<String> todayPublishContentIdList = todayPublishContentList.stream().map(PublishContent::getId).collect(Collectors.toList());
            CountDownLatch cdl = new CountDownLatch(todayPublishContentIdList.size());
            for (String todayPublishContentId : todayPublishContentIdList) {
                pool.submit(() -> {
                    try {
                        result.remove(todayPublishContentId);
                    } finally {
                        cdl.countDown();
                    }
                });
            }
            try {
                cdl.await();
            } catch (InterruptedException e) {
                log.error("TodayPublishStrategy filter error", e);
            }
        }
        filterResult.setContentIds(result);
        log.info("TodayPublishStrategy filter cost time:{}", System.currentTimeMillis() - start);
        return filterResult;
    }

}
